@@ -9,7 +9,7 @@ from tungsten.parsers.field_parse import (
 
 
 class SigmaAldrichFieldMapper(FieldMapper):
-    def getFieldMappings(self, field: SdsQueryFieldName) -> tuple[list[SelectCommand], Callable]:
+    def get_field_mappings(self, field: SdsQueryFieldName) -> tuple[list[SelectCommand], Callable]:
         return {
             # SdsQueryFieldName.META_VERSION: [],
             # SdsQueryFieldName.META_REVISION_DATE: [],
@@ -69,7 +69,8 @@ class SigmaAldrichFieldMapper(FieldMapper):
                 SelectCommand(key="items"),
                 SelectCommand(key="name", where_value=re.compile(r"Company", re.IGNORECASE)),
                 SelectCommand(key="data")
-            ], lambda x: re.match(r"\:?\s*(.*)", "".join(x), re.DOTALL).group(1)),
+            ], lambda x: re.match(r"\:?\s*(.*)",
+                                  " ".join(y.strip() for y in x), re.DOTALL).group(1)),
             SdsQueryFieldName.SUPPLIER_TELEPHONE: ([
                 SelectCommand(key="sections"),
                 SelectCommand(key="title", where_value="IDENTIFICATION"),
@@ -97,7 +98,16 @@ class SigmaAldrichFieldMapper(FieldMapper):
                 SelectCommand(key="name",
                               where_value=re.compile(r"Emergency\sPhone", re.IGNORECASE)),
                 SelectCommand(key="data")
-            ], lambda x: re.match(r"\:?\s*(.*)", "".join(x), re.DOTALL).group(1)),
+            ], lambda x: re.match(
+                    r"\:?\s*(.*)",
+                    "".join(
+                        item \
+                        if item.endswith("-") or item.endswith("/") \
+                        else f"{item} " \
+                        for item in (item.strip() for item in x)
+                    ).rstrip(),
+                    re.DOTALL
+                ).group(1)),
             SdsQueryFieldName.IDENTIFICATION_OTHER: ([
                 SelectCommand(key="sections"),
                 SelectCommand(key="title", where_value="IDENTIFICATION"),
@@ -125,6 +135,19 @@ class SigmaAldrichFieldMapper(FieldMapper):
                 SelectCommand(key="name", where_value=re.compile(r"Signal\sword", re.IGNORECASE)),
                 SelectCommand(key="data")
             ], lambda x: re.search(r"(danger|warning)", "".join(x), re.IGNORECASE).group(1)),
+            SdsQueryFieldName.STATEMENTS: ([
+                SelectCommand(key="sections"),
+                SelectCommand(key="title", where_value="HAZARDS"),
+                SelectCommand(key="subsections")
+            ], lambda x: sorted(  # Forgive me whoever witnesses this bodge
+                [*({match for item in (self.execute_query(x, [
+                    SelectCommand(key="title", where_value="GHS_LABEL_ELEMENTS"),
+                    SelectCommand(key="items")], lambda y: y) or [])
+                    for match in re.findall(r"([HP]\d{3})", item["name"])} |
+                    {match for item in (self.execute_query(x, [
+                        SelectCommand(key="title", where_value="HAZARDS_OTHER"),
+                        SelectCommand(key="items")], lambda y: y) or [])
+                        for match in re.findall(r"([HP]\d{3})", item["name"])})])),
             SdsQueryFieldName.HNOC_HAZARD: ([
                 SelectCommand(key="sections"),
                 SelectCommand(key="title", where_value="HAZARDS"),
